#ifndef AMDIS_INTERPOLRESTRICTMATRIX_H
#define AMDIS_INTERPOLRESTRICTMATRIX_H

#include <map>
#include <vector>
#include <string>
#include "Global.h"
#include "MemoryManager.h"

namespace AMDiS {

  class DOFMatrix;
  class Element;
  template<typename T> class DOFVectorBase;

  /** \brief
   * Defines the data structure for a general interpolation or restriction matrix.
   * Both can be used to map vectors from a fine mesh to a coarser mesh and vice
   * versa.
   */
  class InterpolRestrictMatrix
  {
  public:
    MEMORY_MANAGED(InterpolRestrictMatrix);

    InterpolRestrictMatrix(int numFcts,
			   int numRows,
			   int numCols,
			   bool type0Used);


    /** \brief
     * Creates transposed matrix. Uses memory of original matrix.
     */
    InterpolRestrictMatrix(InterpolRestrictMatrix& matrix,
			   bool transposed);

    virtual ~InterpolRestrictMatrix();

    /** \brief
     *  dest = matrix * src  without assembling global matrix
     */
    void mv(DOFVectorBase<double> *src,
	    DOFVectorBase<double> *dest,
	    int coarseLevel);
    
    void assembleDOFMatrix(DOFMatrix *dest,
			   int coarseLevel);

    void print();

  protected:
    virtual void fillMemory() {};

  protected:
    int numFcts_;
    int numRows_;
    int numCols_;

    std::vector<DegreeOfFreedom> directCopyDOFs_;

    std::map<int, std::map<int, double> > uniqueRowMatrix_;
    std::map<int, std::map<int, double> > uniqueColMatrix_;
    std::map<int, std::map<int, double> > matrix_;

    DegreeOfFreedom *parentDOFs_;
    DegreeOfFreedom *child0DOFs_;
    DegreeOfFreedom *child1DOFs_;

    DegreeOfFreedom **rowDOFPtrs_;
    DegreeOfFreedom **colDOFPtrs_;

    bool type0_used_;

    DegreeOfFreedom **type0_rowDOFPtrs_;
    DegreeOfFreedom **type0_colDOFPtrs_;

    bool transposed_;

    bool initialized_;
  };
}

#endif
