#include "SparseVector.h"
#include "DOFVector.h"
#include "FiniteElemSpace.h"

namespace AMDiS {

  template<>
  void SparseVector<double>::refineInterpol(RCNeighbourList& list, int n)
  {
    (const_cast<BasisFunction*>(this->getFESpace()->getBasisFcts()))->refineInter(this, &list, n);
  }

  template<>
  void SparseVector<double>::coarseRestrict(RCNeighbourList& list, int n)
  {
    (const_cast<BasisFunction*>(this->getFESpace()->getBasisFcts()))->coarseInter(this, &list, n);
  }

  template<>
  void SparseVector<double>::print() 
  {
    FUNCNAME("SparseVector::print()");
    int i, size = static_cast<int>(vec_.size());
    for(i = 0; i < size; i++) {
      MSG("dof %d - %e\n", (*physToLog_)[i], vec_[i]);
    }
  }

  template<>
  void axpy<double>(double a, SparseVector<double>& x, SparseVector<double>& y,
		    std::set<DegreeOfFreedom> &dofSet)
  {
    std::set<DegreeOfFreedom>::iterator dofIt, dofEnd = dofSet.end();
    for(dofIt = dofSet.begin(); dofIt != dofEnd; ++dofIt) {
      y[*dofIt] += a * x[*dofIt];
    }
  }

  template<>
  void mv<double>(DOFMatrix &a, 
		  SparseVector<double> &x,
		  SparseVector<double> &result,
		  std::set<DegreeOfFreedom> &dofSet,
		  bool add)
  {
    //DOFMatrix::Iterator matIt(const_cast<DOFMatrix*>(&a), USED_DOFS);
    std::vector<MatEntry>::iterator rowIt, rowEnd;
  
    DegreeOfFreedom row, col;

    if(!add) result.clear();

    //  for(matIt.reset(); !matIt.end(); ++matIt) {
    std::set<DegreeOfFreedom>::iterator dofIt, dofEnd = dofSet.end();
    for(dofIt = dofSet.begin(); dofIt != dofEnd; ++dofIt) {
      row = *dofIt;
      rowEnd = a[row].end();
      for(rowIt = a[row].begin(); rowIt != rowEnd; ++rowIt) {
	col = rowIt->col;
	if(col == DOFMatrix::UNUSED_ENTRY) continue;
	if(col == DOFMatrix::NO_MORE_ENTRIES) break;
	result[row] += rowIt->entry * const_cast<SparseVector<double>&>(x)[col];
      }
    }
  }

  template<>
  double SparseVector<double>::norm()
  {
    double norm = 0.0;
    std::vector<double>::iterator it, end = vec_.end();
    for(it = vec_.begin(); it != end; ++it) {
      norm += *it * *it;
    }
    return sqrt(norm);
  }

} 
